==> gazpacho_0.5.2.orig.tar.gz: Gazpacho-0.5.2/gazpacho/propertyclass.py <==
# Copyright (C) 2004,2005 by SICEm S.L. and Imendio
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
import gobject

from gazpacho.loader import tags
from gazpacho.l10n import _

# UNICHAR and UINT are internally the same thing but we need to
# handle them as separate. Luckily there is only a Unichar prop in
# Gtk+, which is 'insible_char', so that's why we put it here.
# See http://www.sicem.biz/pipermail/gazpacho/2005-January/000097.html
UNICHAR_PROPERTIES = ('invisible-char', )

class PropertyException(Exception): pass

class PropertyClass(object):
    supported_types = (gobject.TYPE_BOOLEAN, gobject.TYPE_FLOAT,
                       gobject.TYPE_INT, gobject.TYPE_UINT,
                       gobject.TYPE_DOUBLE, gobject.TYPE_STRING,
                       gobject.TYPE_ENUM, gobject.TYPE_FLAGS,
                       gobject.TYPE_OBJECT)
    
    def __init__(self, spec=None):
        from gazpacho import choice
        self._set_default_values()

        # See if it is a custom (non gtk) property
        if not spec:
            self.custom = True
            return
        else:
            self.custom = False

        self.spec = spec
        
        # The id of the property. Like "label" or "xpad" this is
        # a non-translatable string
        self.id = spec.name

        # The type of property from GladePropertyType
        self.type = self._get_type_from_spec(spec)

        if self.type is None:
            raise PropertyException(_('Type Error'))
            
        # The name of the property. Like "Label" or "X Pad" this is
        # a translatable string
        self.name = spec.nick
        if not self.id or not self.name:
            raise PropertyException(_('Failed to create property class from spec'))

        # The tooltip. Currently unimplemented. Not sure if it should go here
        self.tooltip = spec.blurb

        # This default is used when saving the widgets. Only if the value of the
        # property is different from the original default, the prop is saved
        self.original_default = self._get_default_from_spec(spec)

        # This default is used when creating widgets
        # It can be modified by the catalog file
        self.default = self.original_default

        if self.type == gobject.TYPE_ENUM:
            self.choices = choice.list_new_from_spec(spec)
            self._enum_type = spec.value_type
        elif self.type == gobject.TYPE_FLAGS:
            self._enum_type = spec.value_type
        elif self.type == gobject.TYPE_DOUBLE:
            self.parameters = self._get_parameters_numeric(spec)
        elif self.type == gobject.TYPE_OBJECT:
            raise PropertyException(_('Type not suported'))

    def _set_default_values(self):
        # initial values
        self.type = None
        self.id = self.name = self.tooltip = self.default = None
        self.original_default = None
        self.spec = None
        self.disabled = False

        # list of GladeParameter objects. This list provides with an extra
        # set of key-value pairs to specify aspects of this property. Like the
        # number of "VisibleLines" of a text property. Or the range and default
        # values of a spin button entry. Also the default choice for
        # a type == CHOICE
        self.parameters = []

        # list of GladeChoice items. This is only used for propeties of type
        # GLADE_PROPERTY_TYPE_CHOICE and is [] for other poperties.
        self.choices = []

        #  If it is TYPE_ENUM or TYPE_FLAGS, this holds the GType of the enum
        # or flags, otherwise it's 0.
        self._enum_type = 0
        
        # Some properties are optional by nature like default width. It can be
        # set or not set. A default property has a check box in the left that
        # enables/disables de input
        self.optional = False

        # For optional values, what the default is
        self.optional_default = True

        self._visibility_function = None
        
        # A  GladeWidgetClass pointer of objects that we need to set for this
        # widget for example : GtkSpinButton has a Adjustment inside a GtkCombo
        # has an entry inside and a GtkClist which makes a drop dowm menu. This
        # is only valid when the type is object.
        self.child = False
        
        # Common properties go in the common tab
        self.common = False

        # Packing properties go in the packing tab
        self.packing = False

        # If true, this property_class has been "modified" from the the
        # standard property by a xml file
        self.is_modified = False

        # If this property can't be set with g_object_set then we need to
        # implement it inside glade. This is a pointer to the function that
        # can set this property. The functions to work arround this problems
        # are inside gladegtk.py
        self._set_function = None

        # If this property can't be get with g_object_get then we need to
        # implement it inside glade. This is a pointer to the function that
        # can get this property. The functions to work arround this problems
        # are inside gladegtk.py
        self._get_function = None

        # Some properties requires a special widget for editing the property.
        # This function creates the widget.
        self._editor_create_function = None
        self._editor_load_function = None

        # If true, additional i18n metadata can be edited for string
        # properties
        self.is_translatable = True

    def is_visible(self, widget_class):
        if self._visibility_function:
            return self._visibility_function(widget_class)
        return True

    def _get_type_from_spec(self, spec):
        if spec.name == 'name':
            result = None
        elif spec.name == 'user-data':
            result = None
        elif hasattr(spec, 'flags_class'):
            result = gobject.TYPE_FLAGS
        elif hasattr(spec, 'enum_class'):
            result = gobject.TYPE_ENUM
        elif spec.value_type in PropertyClass.supported_types:
            result = spec.value_type
        else:
            result = None

        return result

    def _get_default_from_spec(self, spec):
        if self.type in (gobject.TYPE_ENUM, gobject.TYPE_FLAGS,
                          gobject.TYPE_STRING, gobject.TYPE_INT,
                          gobject.TYPE_FLOAT, gobject.TYPE_DOUBLE,
                          gobject.TYPE_BOOLEAN, gobject.TYPE_UINT):
            return spec.default_value
        else:
            return None
        
    def _get_parameters_numeric(self, pspec):
        return [] # XXX TODO

    def _str_to_enum(self, str):
        type_map = {
            tags.STRING: gobject.TYPE_STRING,
            tags.BOOLEAN: gobject.TYPE_BOOLEAN,
            tags.UNICHAR: gobject.TYPE_UINT,
            tags.UINTEGER: gobject.TYPE_UINT,
            tags.FLOAT: gobject.TYPE_FLOAT,
            tags.INTEGER: gobject.TYPE_INT,
            tags.ENUM: gobject.TYPE_ENUM,
            tags.FLAGS: gobject.TYPE_FLAGS,
            tags.OBJECT: gobject.TYPE_OBJECT
            }
        return type_map.get(str)
    
    def update_from_node(self, xmlnode, widget_class):
        from gazpacho import util, parameter, choice

        # check the id
        prop_id = xmlnode.getAttribute(tags.ID)
        if not prop_id:
            return False

        if util.xml_get_property_boolean(xmlnode, tags.DISABLED, False):
            self.disabled = True
            return True

        # if needed, update the name
        name = xmlnode.getAttribute(tags.NAME)
        if name:
            self.name = name

        # ...the type...
        child = xmlnode.getElementsByTagName(tags.TYPE)
        if child:
            oldtype = util.xml_get_text_from_node(child[0])
            newtype = self._str_to_enum(oldtype)
            if newtype is None:
                return False
            self.type = newtype

        # ...and the tooltip
        child = xmlnode.getElementsByTagName(tags.TOOLTIP)
        if child:
            tooltip = util.xml_get_text_from_node(child[0])
            if tooltip:
                self.tooltip = tooltip

        # Get the visibility function
        child = xmlnode.getElementsByTagName(tags.VISIBILITY_FUNCTION)
        if child:
            try:
                name = util.xml_get_text_from_node(child[0])
                self._visibility_function = getattr(widget_class.module, name)
            except:
                print _('Could not find visibility function %s') % name

        # Get the parameters
        child = xmlnode.getElementsByTagName(tags.PARAMETERS)
        if child:
            self.parameters = parameter.list_new_from_node(child[0])
            self.optional = parameter.get_boolean(self.parameters,
                                                  tags.OPTIONAL, False)

        # Get the choices
        if self.type == gobject.TYPE_ENUM:
            child = xmlnode.getElementsByTagName(tags.ENUMS)
            if child:
                type_name = child[0].getAttribute(tags.ENUM_TYPE)
                prop_type = gobject.type_from_name(type_name)
                self._enum_type = prop_type
                pspecs = [pspec for pspec in \
                          gobject.list_properties(widget_class.type) if \
                          pspec.value_type == prop_type]
                if len(pspecs) != 1:
                    print _('Could not find the property spec for enum type %s') % \
                          type_name
                self.choices = choice.list_new_from_node(child[0], pspecs[0])

        # If the property is an object load it
        if self.type == gobject.TYPE_OBJECT:
            child = xmlnode.getElementsByTagName(tags.GLADE_WIDGET_CLASS)
            if child:
                pass # XXX TODO

        # Get the default
        default = xmlnode.getAttribute(tags.DEFAULT)
        if default:
            if self.type == gobject.TYPE_INT:
                self.default = int(default)
            elif self.type in (gobject.TYPE_FLOAT, gobject.TYPE_DOUBLE):
                self.default = float(default)
            else:
                self.default = default

        # common, optional, etc
        self.common = util.xml_get_property_boolean(xmlnode, tags.COMMON,
                                                    False)
        self.optional = util.xml_get_property_boolean(xmlnode, tags.OPTIONAL,
                                                       False)
        if self.optional:
            self.optional_default = util.xml_get_property_boolean(xmlnode,
                                                                   tags.OPTIONAL_DEFAULT,
                                                                   False)

        self.is_translatable = util.xml_get_property_boolean(xmlnode,
                                                             tags.TRANSLATABLE,
                                                             True)
        
        child = xmlnode.getElementsByTagName(tags.SET_FUNCTION)
        if child:
            try:
                name = util.xml_get_text_from_node(child[0])
                self._set_function = getattr(widget_class.module, name)
            except:
                print _("Unable to get the 'set' function [%s] of the property [%s] of the widget's class [%s] from the module [%s]") % \
                      (name, self.name,
                       widget_class.name,
                       widget_class.module)
        
        child = xmlnode.getElementsByTagName(tags.GET_FUNCTION)
        if child:
            try:
                name = util.xml_get_text_from_node(child[0])
                self._get_function = getattr(widget_class.module, name)
            except:
                print _("Unable to get the 'get' function [%s] of the property [%s] of the widget's class [%s] from the module [%s]")  % \
                      (name,
                       self.name,
                       widget_class.name,
                       widget_class.module)
        

        editor = xmlnode.getElementsByTagName(tags.EDITOR)
        if editor:
            child = editor[0].getElementsByTagName(tags.CREATE_FUNCTION)
            if child:
                try:
                    name = util.xml_get_text_from_node(child[0])
                    self._editor_create_function = getattr(widget_class.module, name)
                except:
                    print _("Unable to get the 'editor-create' function [%s] of the property [%s] of the widget's class [%s] from the module [%s]") % \
                          (name,
                           self.name,
                           widget_class.name,
                           widget_class.module)
            child = editor[0].getElementsByTagName(tags.LOAD_FUNCTION)
            if child:
                try:
                    name = util.xml_get_text_from_node(child[0])
                    self._editor_load_function = getattr(widget_class.module, name)
                except:
                    print _("Unable to get the 'editor load function' function [%s] of the property [%s] of the widget's class [%s] from the module [%s]") % \
                          (name,
                           self.name,
                           widget_class.name,
                           widget_class.module)
                        
        # notify that we changed the property class
        self.is_modified = True
        
        return True

    def __str__(self):
        return "%s" % self.id

    def __repr__(self):
        return "%s" % self.id
    
def type2str(type):
    if type is not None:
        return gobject.type_name(type)
    else:
        return _('Error')

==> gazpacho_0.5.3.orig.tar.gz: Gazpacho-0.5.3/gazpacho/propertyclass.py <==
# Copyright (C) 2004,2005 by SICEm S.L. and Imendio AB
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
import gobject
import gtk

from gazpacho.loader import tags

def get_type_from_spec(spec):
    if getattr(spec, 'flags_class', None) is not None:
        result = gobject.TYPE_FLAGS
    elif getattr(spec, 'enum_class', None) is not None:
        result = gobject.TYPE_ENUM
    elif spec.value_type in PropertyClass.supported_types:
        result = spec.value_type
    else:
        result = gobject.TYPE_OBJECT

    return result


# UNICHAR and UINT are internally the same thing but we need to
# handle them as separate. Luckily there is only a Unichar prop in
# Gtk+, which is 'invisible_char', so that's why we put it here.
# See http://www.sicem.biz/pipermail/gazpacho/2005-January/000097.html
UNICHAR_PROPERTIES = ('invisible-char', )

class PropertyException(Exception): pass

class PropertyClass(object):
    # supported types for the 'value_type' field of a pspec property
    supported_types = (gobject.TYPE_BOOLEAN, gobject.TYPE_FLOAT,
                       gobject.TYPE_INT, gobject.TYPE_UINT,
                       gobject.TYPE_DOUBLE, gobject.TYPE_STRING,
                       gtk.Adjustment.__gtype__)
    
    def __init__(self, spec=None):
        self._set_default_values()

        # See if it is a custom (non gtk) property
        if not spec:
            self.custom = True
            return
        else:
            self.custom = False

        self._update_from_pspec(spec)
            
    def _set_default_values(self):
        # initial values
        self.type = None
        self.id = self.name = self.tooltip = self.default = None
        self.original_default = None
        self.spec = None
        self.disabled = False

        # list of GladeParameter objects. This list provides with an extra
        # set of key-value pairs to specify aspects of this property. Like the
        # number of "VisibleLines" of a text property. Or the range and default
        # values of a spin button entry. Also the default choice for
        # a type == CHOICE
        self.parameters = []

        # list of GladeChoice items. This is only used for propeties of type
        # GLADE_PROPERTY_TYPE_CHOICE and is [] for other poperties.
        self.choices = []

        #  If it is TYPE_ENUM or TYPE_FLAGS, this holds the GType of the enum
        # or flags, otherwise it's 0.
        self._enum_type = 0
        
        # Some properties are optional by nature like default width. It can be
        # set or not set. A default property has a check box in the left that
        # enables/disables de input
        self.optional = False

        # For optional values, what the default is
        self.optional_default = True

        self._visibility_function = None
        
        # A  GladeWidgetClass pointer of objects that we need to set for this
        # widget for example : GtkSpinButton has a Adjustment inside a GtkCombo
        # has an entry inside and a GtkClist which makes a drop dowm menu. This
        # is only valid when the type is object.
        self.child = False
        
        # Common properties go in the common tab
        self.common = False

        # Packing properties go in the packing tab
        self.packing = False

        # If true, this property_class has been "modified" from the the
        # standard property by a xml file
        self.is_modified = False

        # If properties can't use the standard g_object_get and g_object_set
        # functions we need to implement our own functions for that. These
        # are function objects that can get this property.
        self._set_function = None
        self._get_function = None

        # Some properties need custom load and save functions
        self._load_function = None
        self._save_function = None
        
        # Some properties requires a special widget for editing the property.
        # These functions create and update the widget
        self._editor_create_function = None
        self._editor_update_function = None

        # If true, additional i18n metadata can be edited for string
        # properties
        self.is_translatable = True

        # some methods are delegated to this adaptor, in case we have one
        self.adaptor = None

        # if editable is false it doesn't appear in the Editor. This is useful
        # for some properties we wan't to store but that are too difficult
        # for the user to understand (e.g. use-stock)
        self.editable = True

    def _update_from_pspec(self, spec):
        from gazpacho import choice

        self.spec = spec
        
        # The id of the property. Like "label" or "xpad" this is
        # a non-translatable string
        if not self.id:
            self.id = spec.name

        # The type of property from GladePropertyType
        self.type = get_type_from_spec(spec)

        # The name of the property. Like "Label" or "X Pad" this is
        # a translatable string
        self.name = spec.nick
        if not self.id or not self.name:
            msg = 'Failed to create property class from spec'
            raise PropertyException, msg

        # The tooltip. Currently unimplemented. Not sure if it should go here
        self.tooltip = spec.blurb

        # This default is used when saving the widgets. Only if the value of
        # the property is different from the original default, the prop is
        # saved
        self.original_default = self._get_default_from_spec(spec)

        # This default is used when creating widgets
        # It can be modified by the catalog file
        self.default = self.original_default

        if self.type == gobject.TYPE_ENUM:
            self.choices = choice.list_new_from_spec(spec)
            self._enum_type = spec.value_type
        elif self.type == gobject.TYPE_FLAGS:
            self._enum_type = spec.value_type
        elif self.type == gobject.TYPE_DOUBLE:
            self.parameters = self._get_parameters_numeric(spec)

        if self.id == 'name':
            self.is_translatable = False
        
    def is_visible(self, widget_class):
        if self._visibility_function:
            return self._visibility_function(widget_class)
        return True

    def _get_default_from_spec(self, spec):
        if self.type in (gobject.TYPE_ENUM, gobject.TYPE_FLAGS,
                          gobject.TYPE_STRING, gobject.TYPE_INT,
                          gobject.TYPE_FLOAT, gobject.TYPE_DOUBLE,
                          gobject.TYPE_BOOLEAN, gobject.TYPE_UINT):
            return spec.default_value
        else:
            return None
        
    def _get_parameters_numeric(self, pspec):
        return [] # XXX TODO

    def _str_to_enum(self, str):
        type_map = {
            tags.STRING: gobject.TYPE_STRING,
            tags.BOOLEAN: gobject.TYPE_BOOLEAN,
            tags.UNICHAR: gobject.TYPE_UINT,
            tags.UINTEGER: gobject.TYPE_UINT,
            tags.FLOAT: gobject.TYPE_FLOAT,
            tags.INTEGER: gobject.TYPE_INT,
            tags.ENUM: gobject.TYPE_ENUM,
            tags.FLAGS: gobject.TYPE_FLAGS,
            tags.OBJECT: gobject.TYPE_OBJECT
            }
        return type_map.get(str)
    
    def update_from_node(self, xmlnode, widget_class):
        from gazpacho import util, parameter, choice, function

        # check the id
        prop_id = xmlnode.getAttribute(tags.ID)
        if not prop_id:
            return False

        # is it disabled?
        if util.xml_get_property_boolean(xmlnode, tags.DISABLED, False):
            self.disabled = True
            return True

        # is it editable?
        if not util.xml_get_property_boolean(xmlnode, tags.EDITABLE, True):
            self.editable = False
        
        # if needed, update the name
        name = xmlnode.getAttribute(tags.NAME)
        if name:
            self.name = name

        # ...the type...
        child = xmlnode.getElementsByTagName(tags.TYPE)
        if child:
            oldtype = util.xml_get_text_from_node(child[0])
            newtype = self._str_to_enum(oldtype)
            if newtype is None:
                return False
            self.type = newtype

        # ...and the tooltip
        child = xmlnode.getElementsByTagName(tags.TOOLTIP)
        if child:
            tooltip = util.xml_get_text_from_node(child[0])
            if tooltip:
                self.tooltip = tooltip

        # Get the parameters
        child = xmlnode.getElementsByTagName(tags.PARAMETERS)
        if child:
            self.parameters = parameter.list_new_from_node(child[0])
            self.optional = parameter.get_boolean(self.parameters,
                                                  tags.OPTIONAL, False)

        # Get the choices
        if self.type == gobject.TYPE_ENUM:
            child = xmlnode.getElementsByTagName(tags.ENUMS)
            if child:
                type_name = child[0].getAttribute(tags.ENUM_TYPE)
                prop_type = gobject.type_from_name(type_name)
                self._enum_type = prop_type
                pspecs = [pspec for pspec in \
                          gobject.list_properties(widget_class.type) if \
                          pspec.value_type == prop_type]
                if len(pspecs) != 1:
                    print _('Could not find the property spec for enum type %s') % \
                          type_name
                self.choices = choice.list_new_from_node(child[0], pspecs[0])

        # If the property is an object load it
        if self.type == gobject.TYPE_OBJECT:
            child = xmlnode.getElementsByTagName(tags.GLADE_WIDGET_CLASS)
            if child:
                pass # XXX TODO

        # Get the default
        default = xmlnode.getAttribute(tags.DEFAULT)
        if default:
            if self.type == gobject.TYPE_INT:
                self.default = int(default)
            elif self.type in (gobject.TYPE_FLOAT, gobject.TYPE_DOUBLE):
                self.default = float(default)
            else:
                self.default = default

        # common, optional, etc
        self.common = util.xml_get_property_boolean(xmlnode, tags.COMMON,
                                                    False)
        self.optional = util.xml_get_property_boolean(xmlnode, tags.OPTIONAL,
                                                       False)
        if self.optional:
            self.optional_default = util.xml_get_property_boolean(xmlnode,
                                                                   tags.OPTIONAL_DEFAULT,
                                                                   False)

        self.is_translatable = util.xml_get_property_boolean(xmlnode,
                                                             tags.TRANSLATABLE,
                                                             True)

        # adaptor
        adaptor_class_name = xmlnode.getAttribute(tags.ADAPTOR_CLASS)
        library = widget_class.library
        self.adaptor = library.get_property_adaptor_class(widget_class.name,
                                                          prop_id,
                                                          self.type,
                                                          adaptor_class_name)
        
        functions = util.xml_get_elements_one_level(xmlnode, tags.FUNCTION)

        for func in functions:
            func_name = func.getAttribute(tags.NAME)

            if func_name == tags.GET:
                function.override(func, widget_class, '_get_function',
                                  property_class=self)
            elif func_name == tags.SET:
                function.override(func, widget_class, '_set_function',
                                  property_class=self)
            elif func_name == tags.VISIBILITY:
                function.override(func, widget_class, '_visibility_function',
                                  property_class = self)
            elif func_name == tags.SAVE:
                function.override(func, widget_class, '_save_function',
                                  property_class = self)
                
        editor = xmlnode.getElementsByTagName(tags.EDITOR)
        if editor:
            functions = util.xml_get_elements_one_level(editor.item(0),
                                                        tags.FUNCTION)

            for func in functions:
                func_name = func.getAttribute(tags.NAME)

                if func_name == tags.CREATE:
                    function.override(func, widget_class,
                                      '_editor_create_function', self,
                                      'create_editor')
                elif func_name == tags.UPDATE:
                    function.override(func, widget_class,
                                      '_editor_update_function', self,
                                      'update_editor')
                else:
                    print "Unknown editor function: (%s) %s" % (self.name,
                                                                func_name)

                        
        # notify that we changed the property class
        self.is_modified = True
        
        return True

    def __str__(self):
        return "%s" % self.id

    def __repr__(self):
        return "%s" % self.id
    
def type2str(type):
    if type is not None:
        return gobject.type_name(type)
    else:
        return _('Error')

class SubPropertyClass(PropertyClass):
    def __init__(self):
        PropertyClass.__init__(self)

    def _set_sub_property (self, context, object, value):
        sub_name = self.id.split('.')[-1]
        
        for prop_name in self.id.split('.')[:-1]:
            obj = object.get_property(prop_name)

        obj.set_property(sub_name, value)
        
    def _get_sub_property (self, context, obj):
        sub_name = self.id.split('.')[-1]
        
        for prop_name in self.id.split('.')[:-1]:
            obj = obj.get_property(prop_name)
            
        value = obj.get_property(sub_name)
        
        return value
        
    def update_from_node(self, xmlnode, widget_class):
        prop_type = widget_class.type

        self._get_function = self._get_sub_property
        self._set_function = self._set_sub_property

        found_prop = None
        for prop_name in self.id.split('.'):
            # XXX: For the first property we should probably use
            # the custom get function
            properties = gobject.list_properties(prop_type)
            
            for prop in properties:
                if prop.name == prop_name:
                    found_prop = prop
                    prop_type = prop.value_type
                    break
            else:
                msg = _('Could not find sub property %s') % prop_name
                raise PropertyException, msg

        self._update_from_pspec(found_prop)
        
        return super(SubPropertyClass, self).update_from_node(xmlnode,
                                                              widget_class)
